/*
 * FreeModbus Libary: STM32F1xx Port with FreeRTOS
 * 
 */

/* ----------------------- System includes ----------------------------------*/
#include "assert.h"


/* ----------------------- STM32F1xx includes ----------------------------------*/
/*#include "gpio.h"
#include "eic.h"
#include "uart.h"
#include "tim.h"*/

/* ----------------------- Platform includes --------------------------------*/
#include "port.h"
#include "common.h"

/* ----------------------- Modbus includes ----------------------------------*/
#include "mb.h"
#include "mbport.h"

/* ----------------------- Defines ------------------------------------------*/
#define USARTx				USART2
#define USARTirq			USART2_IRQn
#define GPIOx_TX			GPIOA
#define PIN_TX				GPIO_Pin_2
#define GPIOx_RX			GPIOA
#define PIN_RX				GPIO_Pin_3
#define GPIOx_RTS			GPIOA
#define PIN_RTS				GPIO_Pin_1
#define USARTx_IRQHandler	USART2_IRQHandler

/* ----------------------- Static functions ---------------------------------*/

/* ----------------------- Start implementation -----------------------------*/

static volatile BOOL txen = 0;
static volatile BOOL rxen = 0;

//high=tx, low=rx
#define DIR_TX	1
#define DIR_RX	0
static inline void dirOutput(int hi)
{
	if(hi) GPIOx_RTS->BSRR = PIN_RTS; else GPIOx_RTS->BRR = PIN_RTS;
}


BOOL
xMBPortSerialInit( UCHAR ucPort, ULONG ulBaudRate, UCHAR ucDataBits, eMBParity eParity )
{
	xprintf("xMBPortSerialInit called...\n");
	
	(void)ucPort;
	
    BOOL            xResult = TRUE;
	
	//enable clockz
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);	
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
	
	GPIO_InitTypeDef  GPIO_InitStructure;
	
	//dir (rts) pin init
	GPIO_InitStructure.GPIO_Pin = PIN_RTS;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOx_RTS, &GPIO_InitStructure);
	
	//tx pin init
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	
	//rx pin init
	GPIO_InitStructure.GPIO_Pin             = GPIO_Pin_3;
	GPIO_InitStructure.GPIO_Mode            = GPIO_Mode_IN_FLOATING;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	
	//initialize USART
	USART_InitTypeDef USART_InitStructure;
	USART_InitStructure.USART_BaudRate = ulBaudRate;
	switch(ucDataBits)
	{
		case 8:
			USART_InitStructure.USART_WordLength = USART_WordLength_8b;
			break;
		case 9:
			USART_InitStructure.USART_WordLength = USART_WordLength_9b;
			break;
		default:
			USART_InitStructure.USART_WordLength = USART_WordLength_8b;
			xResult = FALSE;
	}
	
	USART_InitStructure.USART_StopBits = USART_StopBits_1;
	
	switch(eParity)
	{
		case MB_PAR_NONE:
			USART_InitStructure.USART_Parity = USART_Parity_No;
			break;
		case MB_PAR_ODD:
			USART_InitStructure.USART_Parity = USART_Parity_Odd;
			break;
		case MB_PAR_EVEN:
			USART_InitStructure.USART_Parity = USART_Parity_Even;
			break;
	}
	USART_InitStructure.USART_HardwareFlowControl =USART_HardwareFlowControl_None;
	USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

	/* Configure USARTx */
	USART_Init(USART2, &USART_InitStructure);

	/* Enable the USARTx */
	USART_Cmd(USART2, ENABLE);
	
	NVIC_InitTypeDef NVIC_InitStructure;
	NVIC_InitStructure.NVIC_IRQChannel = USARTirq;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 9;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init( &NVIC_InitStructure );
	
	USART_ITConfig(USARTx, USART_IT_TC, ENABLE);
	
    return xResult;
}

void
vMBPortSerialEnable( BOOL xRxEnable, BOOL xTxEnable )
{
	if(xRxEnable)
	{
		if(rxen==0)
		{
			rxen=1;
			USART_ClearITPendingBit(USARTx, USART_IT_RXNE);
			USART_ITConfig(USARTx, USART_IT_RXNE, ENABLE);
		}
	}
	else
	{
		if(rxen)
		{
			rxen=0;
			USART_ITConfig(USARTx, USART_IT_RXNE, DISABLE);
			USART_ClearITPendingBit(USARTx, USART_IT_RXNE);
		}
	}
	
	if(xTxEnable)
	{
		if(txen==0)
		{
			txen=1;
			USART_ClearITPendingBit(USARTx, USART_IT_RXNE);
			USART_ITConfig(USARTx, USART_IT_TXE, ENABLE);
		}
	}
	else
	{
		if(txen)
		{
			txen=0;
			USART_ITConfig(USARTx, USART_IT_TXE, DISABLE);
			USART_ClearITPendingBit(USARTx, USART_IT_RXNE);
		}
	}
}

BOOL
xMBPortSerialPutByte( CHAR ucByte )
{
	//while(USART_GetFlagStatus(USARTx, USART_FLAG_TXE) == RESET);
	dirOutput(DIR_TX);
	USART_SendData(USARTx, (uint16_t)ucByte);
	return TRUE;
}

BOOL
xMBPortSerialGetByte( CHAR * pucByte )
{
	*pucByte = USART_ReceiveData(USARTx);
	return TRUE;
}


void
USART2_IRQHandler( void )
{
	//debug_chr('I');

	if(USART_GetITStatus(USARTx, USART_IT_TC) != RESET)
	{
		dirOutput(DIR_RX);
		USART_ClearITPendingBit(USARTx, USART_IT_TC);
	}
	
	if(USART_GetITStatus(USARTx, USART_IT_RXNE) != RESET)
	{
		pxMBFrameCBByteReceived();
		//debug_chr('r');
		USART_ClearITPendingBit(USARTx, USART_IT_RXNE);
	}
	
	if(USART_GetITStatus(USARTx, USART_IT_TXE) != RESET)
	{
		pxMBFrameCBTransmitterEmpty();
		//debug_chr('t');
		//USART_ClearITPendingBit(USARTx, USART_IT_TXE);
	}
}
