/*
 * FreeModbus Libary: STR71x Port
 * Copyright (C) 2006 Christian Walter <wolti@sil.at>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * File: $Id: porttimer.c,v 1.6 2006/05/14 21:54:16 wolti Exp $
 */

/* ----------------------- System includes ----------------------------------*/
#include "assert.h"

/* ----------------------- Platform includes --------------------------------*/
#include "port.h"

/* ----------------------- Modbus includes ----------------------------------*/
#include "mb.h"
#include "mbport.h"

#include "common.h"

#include "leds.h"

static volatile uint32_t timen = 0;

#define TOUT_TIM			TIM2
#define TOUT_RCC_PERIPH		RCC_APB1Periph_TIM2
#define TOUT_CLK_SRC		APB1
#define TOUT_IRQN			TIM2_IRQn
#define TOUT_IRQ_HANDLER	TIM2_IRQHandler

BOOL
xMBPortTimersInit( uint16_t usTim1Timerout50us )
{
	TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
	RCC_APB1PeriphClockCmd(TOUT_RCC_PERIPH , ENABLE);
	TIM_DeInit(TOUT_TIM);
	TIM_TimeBaseStructure.TIM_Period=usTim1Timerout50us + 1;	//+5 OK
	
	RCC_ClocksTypeDef RCC_ClocksStatus;
	RCC_GetClocksFreq(&RCC_ClocksStatus);
	
	#if (TOUT_CLK_SRC == APB1)
	uint32_t f_cpu = RCC_ClocksStatus.PCLK1_Frequency;
	#else
	uint32_t f_cpu = RCC_ClocksStatus.PCLK2_Frequency;
	#endif
	TIM_TimeBaseStructure.TIM_Prescaler= f_cpu / 10000;
	
	//xprintf("xMBPortTimersInit: prescaler value = %d\n",TIM_TimeBaseStructure.TIM_Prescaler);
	
	TIM_TimeBaseStructure.TIM_ClockDivision=TIM_CKD_DIV1;
	TIM_TimeBaseStructure.TIM_CounterMode=TIM_CounterMode_Up;
	TIM_TimeBaseInit(TOUT_TIM, &TIM_TimeBaseStructure);
	TIM_ClearFlag(TOUT_TIM, TIM_FLAG_Update);
	TIM_ITConfig(TOUT_TIM,TIM_IT_Update,ENABLE);
	
	NVIC_InitTypeDef NVIC_InitStructure;
	NVIC_InitStructure.NVIC_IRQChannel = TOUT_IRQN;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 10;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	
	TIM_ClearITPendingBit(TOUT_TIM,TIM_IT_Update);
	
	TIM_SetCounter(TOUT_TIM,0);
	
	NVIC_Init(&NVIC_InitStructure);
	
	TIM_Cmd(TOUT_TIM, DISABLE);
	
	timen = 0;
	
    return 1;
}


void TOUT_IRQ_HANDLER(void)
{
	//xprintf("*");
	if( TIM_GetITStatus(TOUT_TIM , TIM_IT_Update) != RESET )
	{
		TIM_ClearITPendingBit(TOUT_TIM,TIM_IT_Update);
		
		if(timen==0)
		{
			xprintf("T2 error: IRQ @ disable\n");
		}
		else
		{
			//debug_chr('*');
			pxMBPortCBTimerExpired();
		}
	}
}




inline void
vMBPortTimersEnable()
{
	TIM_SetCounter(TOUT_TIM,0);
	if(timen==0)
	{
		timen=1;
		TIM_Cmd(TOUT_TIM, ENABLE);
	}
}

inline void
vMBPortTimersDisable()
{
	if(timen)
	{
		timen=0;
		TIM_Cmd(TOUT_TIM, DISABLE);
		TIM_ClearITPendingBit(TOUT_TIM,TIM_IT_Update);
	}
}
