#include "stm32f10x.h"
#include "stm32f10x_it.h"
#include "stm32f10x_conf.h"

#include "dbgu.h"

#define USARTx USART1


//set to nonzero for full implementation (more code)
#define FULL						0


static void initUsartPins(void)
{
	GPIO_InitTypeDef  GPIO_InitStructure;
	
	//tx pin init
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	
	//rx pin init
	GPIO_InitStructure.GPIO_Pin             = GPIO_Pin_10;
	GPIO_InitStructure.GPIO_Mode            = GPIO_Mode_IN_FLOATING;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
}

void debug_init_default(void)
{
	//enable clockz
	RCC_APB1PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);	
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
	
	initUsartPins();
	
	//initialize USART
	USART_InitTypeDef USART_InitStructure;
	USART_InitStructure.USART_BaudRate = 115200;
	USART_InitStructure.USART_WordLength = USART_WordLength_8b;
	USART_InitStructure.USART_StopBits = USART_StopBits_1;
	USART_InitStructure.USART_Parity = USART_Parity_No;
	USART_InitStructure.USART_HardwareFlowControl =USART_HardwareFlowControl_None;
	USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

	/* Configure USARTx */
	USART_Init(USARTx, &USART_InitStructure);

	/* Enable the USARTx */
	USART_Cmd(USARTx, ENABLE);
}

int debug_test(void)
{
	return ( USART_GetFlagStatus(USARTx, USART_FLAG_RXNE) == RESET ) ? 0 : 1;
}


//send chr via UART (platform dependent)
void debug_chr(char chr)
{
	while(USART_GetFlagStatus(USARTx, USART_FLAG_TXE) == RESET) { ; }
	USART_SendData(USARTx, (uint16_t)chr);
}

int putChar(int ch)
{
	debug_chr(ch);
	return ch;
}



//returns ascii value of last char received
//returns 0 if no char was received since last debug_inkey call
//(platform dependent)
char debug_inkey(void)
{
	if(USART_GetFlagStatus(USARTx, USART_FLAG_RXNE) == RESET)
		return(0);
	else
		return (unsigned char)USART_ReceiveData(USARTx);
}

//halts program/task execution until char is received
//(platform dependent)
char debug_waitkey(void)
{
	while(USART_GetFlagStatus(USARTx, USART_FLAG_RXNE) == RESET) { ; }
	return (unsigned char)USART_ReceiveData(USARTx);
}

