#include "common.h"
#include "pwm.h"
#include "pwm_lut.h"


/*
Pinout

STM32 PIN	TIM/CH	REMAP	CHANNEL

PC6			T3/CH1		y	0
PC7			T3/CH2		y	1
PC8			T3/CH3		y	2
PC9			T3/CH4		y	3
PB6			T4/CH1		n	4
PB7			T4/CH2		n	5
PB8			T4/CH3		n	6
PB9			T4/CH4		n	7
PA8			T1/CH1		n	8
PA11		T1/CH4		n	9

*/


static const uint16_t PERIOD = 0xFFFF;
static const uint16_t PRESCALER = 0x0001;

#define PWM_INVERT	0


static void tim1Init(void)
{
	// ---- TIM1 ----
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM1, ENABLE);

	RCC_ClocksTypeDef RCC_ClocksStatus;
	RCC_GetClocksFreq(&RCC_ClocksStatus);
	

    TIM_TimeBaseInitTypeDef timerInitStructure;
    timerInitStructure.TIM_Prescaler = PRESCALER;
    timerInitStructure.TIM_CounterMode = TIM_CounterMode_Up;
    timerInitStructure.TIM_Period = PERIOD;
    timerInitStructure.TIM_ClockDivision = TIM_CKD_DIV1;
    timerInitStructure.TIM_RepetitionCounter = 0;
    TIM_TimeBaseInit(TIM1, &timerInitStructure);
	
    TIM_OCInitTypeDef outputChannelInit;
    outputChannelInit.TIM_OCMode = TIM_OCMode_PWM2;
    outputChannelInit.TIM_Pulse = 0;
    outputChannelInit.TIM_OutputState = TIM_OutputState_Enable;
    outputChannelInit.TIM_OutputNState = TIM_OutputState_Disable;
	outputChannelInit.TIM_OCIdleState = TIM_OCIdleState_Set;
	outputChannelInit.TIM_OCNIdleState = TIM_OCNIdleState_Set;
	outputChannelInit.TIM_OCPolarity = TIM_OCPolarity_Low;
	outputChannelInit.TIM_OCNPolarity = TIM_OCNPolarity_Low;

    TIM_OC1Init(TIM1, &outputChannelInit);
    TIM_OC4Init(TIM1, &outputChannelInit);
    TIM_OC1PreloadConfig(TIM1, TIM_OCPreload_Enable);
    TIM_OC4PreloadConfig(TIM1, TIM_OCPreload_Enable);
	
	TIM_ARRPreloadConfig(TIM1,ENABLE);
	
    TIM_Cmd(TIM1, ENABLE);
	
	TIM_CtrlPWMOutputs(TIM1, ENABLE);
	
	// ---- IO init ----
	GPIO_InitTypeDef GPIO_InitStructure;
	GPIO_InitStructure.GPIO_Pin             = GPIO_Pin_8 | GPIO_Pin_11;
	GPIO_InitStructure.GPIO_Speed           = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode            = GPIO_Mode_AF_PP;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
}

static void tim3Init(void)
{
	// ---- TIM3 ----
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);

	RCC_ClocksTypeDef RCC_ClocksStatus;
	RCC_GetClocksFreq(&RCC_ClocksStatus);
	

    TIM_TimeBaseInitTypeDef timerInitStructure;
    timerInitStructure.TIM_Prescaler = PRESCALER;
    timerInitStructure.TIM_CounterMode = TIM_CounterMode_Up;
    timerInitStructure.TIM_Period = PERIOD;
    timerInitStructure.TIM_ClockDivision = TIM_CKD_DIV1;
    timerInitStructure.TIM_RepetitionCounter = 0;
    TIM_TimeBaseInit(TIM3, &timerInitStructure);
	
    TIM_OCInitTypeDef outputChannelInit;
    outputChannelInit.TIM_OCMode = TIM_OCMode_PWM1;
    outputChannelInit.TIM_Pulse = 0;
    outputChannelInit.TIM_OutputState = TIM_OutputState_Enable;
    outputChannelInit.TIM_OCPolarity = TIM_OCPolarity_High;

    TIM_OC1Init(TIM3, &outputChannelInit);
    TIM_OC2Init(TIM3, &outputChannelInit);
    TIM_OC3Init(TIM3, &outputChannelInit);
    TIM_OC4Init(TIM3, &outputChannelInit);
    TIM_OC1PreloadConfig(TIM3, TIM_OCPreload_Enable);
    TIM_OC2PreloadConfig(TIM3, TIM_OCPreload_Enable);
    TIM_OC3PreloadConfig(TIM3, TIM_OCPreload_Enable);
    TIM_OC4PreloadConfig(TIM3, TIM_OCPreload_Enable);
	
	TIM_ARRPreloadConfig(TIM3,ENABLE);
    TIM_Cmd(TIM3, ENABLE);
	
	// ---- IO init ----
	GPIO_PinRemapConfig(GPIO_FullRemap_TIM3,ENABLE);
	GPIO_InitTypeDef GPIO_InitStructure;
	GPIO_InitStructure.GPIO_Pin             = GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9;
	GPIO_InitStructure.GPIO_Speed           = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode            = GPIO_Mode_AF_PP;
	GPIO_Init(GPIOC, &GPIO_InitStructure);
}


static void tim4Init(void)
{
	// ---- TIM4 ----
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM4, ENABLE);

	RCC_ClocksTypeDef RCC_ClocksStatus;
	RCC_GetClocksFreq(&RCC_ClocksStatus);
	

    TIM_TimeBaseInitTypeDef timerInitStructure;
    timerInitStructure.TIM_Prescaler = PRESCALER;
    timerInitStructure.TIM_CounterMode = TIM_CounterMode_Up;
    timerInitStructure.TIM_Period = PERIOD;
    timerInitStructure.TIM_ClockDivision = TIM_CKD_DIV1;
    timerInitStructure.TIM_RepetitionCounter = 0;
    TIM_TimeBaseInit(TIM4, &timerInitStructure);
	
    TIM_OCInitTypeDef outputChannelInit;
    outputChannelInit.TIM_OCMode = TIM_OCMode_PWM1;
    outputChannelInit.TIM_Pulse = 0;
    outputChannelInit.TIM_OutputState = TIM_OutputState_Enable;
    outputChannelInit.TIM_OCPolarity = TIM_OCPolarity_High;

    TIM_OC1Init(TIM4, &outputChannelInit);
    TIM_OC2Init(TIM4, &outputChannelInit);
    TIM_OC3Init(TIM4, &outputChannelInit);
    TIM_OC4Init(TIM4, &outputChannelInit);
    TIM_OC1PreloadConfig(TIM4, TIM_OCPreload_Enable);
    TIM_OC2PreloadConfig(TIM4, TIM_OCPreload_Enable);
    TIM_OC3PreloadConfig(TIM4, TIM_OCPreload_Enable);
    TIM_OC4PreloadConfig(TIM4, TIM_OCPreload_Enable);
	
	TIM_ARRPreloadConfig(TIM4,ENABLE);
    TIM_Cmd(TIM4, ENABLE);
	
	// ---- IO init ----
	GPIO_InitTypeDef GPIO_InitStructure;
	GPIO_InitStructure.GPIO_Pin             = GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9;
	GPIO_InitStructure.GPIO_Speed           = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode            = GPIO_Mode_AF_PP;
	GPIO_Init(GPIOB, &GPIO_InitStructure);
}





void pwmInit(void)
{
	tim3Init();
	tim4Init();
	tim1Init();
	
	for(uint32_t chan=0;chan<10;chan++)
	{
		pwmSet(chan,150);
	}
}


void pwmSet(uint8_t channel, uint8_t value)
{
	uint16_t finalValue = PWM_LUT[value];
	
	switch(channel)
	{
		case 0:
			TIM3->CCR1 = finalValue;
			break;
		case 1:
			TIM3->CCR2 = finalValue;
			break;
		case 2:
			TIM3->CCR3 = finalValue;
			break;
		case 3:
			TIM3->CCR4 = finalValue;
			break;
		case 4:
			TIM4->CCR1 = finalValue;
			break;
		case 5:
			TIM4->CCR2 = finalValue;
			break;
		case 6:
			TIM4->CCR3 = finalValue;
			break;
		case 7:
			TIM4->CCR4 = finalValue;
			break;
		case 8:
			TIM1->CCR1 = finalValue;
			break;
		case 9:
			TIM1->CCR4 = finalValue;
			break;
	}
}

