#include "common.h"

#include "spp.h"

#define USART_BAUDRATE		19200

//compatibility defs
#define USARTx				USART2
#define USARTirq			USART2_IRQn
#define GPIOx_TX			GPIOA
#define PIN_TX				GPIO_Pin_2
#define GPIOx_RX			GPIOA
#define PIN_RX				GPIO_Pin_3
#define GPIOx_RTS			GPIOA
#define PIN_RTS				GPIO_Pin_1
#define USARTx_IRQHandler	USART2_IRQHandler


//high=tx, low=rx
#define DIR_TX	1
#define DIR_RX	0
static inline void dirOutput(int hi)
{
	if(hi) GPIOx_RTS->BSRR = PIN_RTS; else GPIOx_RTS->BRR = PIN_RTS;
}


static void serialInit(void)
{
	//enable clockz
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);	
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
	
	GPIO_InitTypeDef  GPIO_InitStructure;
	
	//dir (rts) pin init
	GPIO_InitStructure.GPIO_Pin = PIN_RTS;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOx_RTS, &GPIO_InitStructure);
	
	//tx pin init
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	
	//rx pin init
	GPIO_InitStructure.GPIO_Pin             = GPIO_Pin_3;
	GPIO_InitStructure.GPIO_Mode            = GPIO_Mode_IN_FLOATING;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	
	//initialize USART
	USART_InitTypeDef USART_InitStructure;
	USART_InitStructure.USART_BaudRate = 19200;
	USART_InitStructure.USART_WordLength = USART_WordLength_8b;
	USART_InitStructure.USART_StopBits = USART_StopBits_1;
	USART_InitStructure.USART_Parity = USART_Parity_No;
	USART_InitStructure.USART_HardwareFlowControl =USART_HardwareFlowControl_None;
	USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

	/* Configure USARTx */
	USART_Init(USART2, &USART_InitStructure);

	/* Enable the USARTx */
	USART_Cmd(USART2, ENABLE);
	
	NVIC_InitTypeDef NVIC_InitStructure;
	NVIC_InitStructure.NVIC_IRQChannel = USARTirq;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x03;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x03;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init( &NVIC_InitStructure );
	
	//USART_ITConfig(USARTx, USART_IT_TC, ENABLE);
	USART_ITConfig(USARTx, USART_IT_RXNE, ENABLE);
	//USART_ITConfig(USARTx, USART_IT_TXE, ENABLE);
}





//tx byte
static void txb(uint8_t b)
{
	dirOutput(DIR_TX);
	USART_SendData(USARTx, (uint16_t)b);
	while(USART_GetFlagStatus(USARTx, USART_FLAG_TXE) == RESET);
	USART_ITConfig(USARTx, USART_IT_TC, ENABLE);
}


//input: packet to be sent
//returns: 0 on error, nonzero if OK
uint8_t sppTx(T_sppPacket* pPacket)
{
	return(txPacket_core(pPacket));
}

//enable rx interrupt
static void rxIntEna(void)
{
	USART_ITConfig(USARTx, USART_IT_RXNE, ENABLE);
}

//disable rx interrupt
static void rxIntDis(void)
{
	USART_ITConfig(USARTx, USART_IT_RXNE, DISABLE);
}

//rx packet
uint8_t sppRx(T_sppPacket* pPacket)
{
	return (rxPacket_core(pPacket));
}


void USARTx_IRQHandler(void)
{
	if(USART_GetITStatus(USARTx, USART_IT_TC) != RESET)
	{
		//debug_chr('c');
		dirOutput(DIR_RX);
		USART_ClearITPendingBit(USARTx, USART_IT_TC);
		USART_ITConfig(USARTx, USART_IT_TC, DISABLE);
	}
	
	if(USART_GetITStatus(USARTx, USART_IT_RXNE) != RESET)
	{
		uint8_t chr = USART_ReceiveData(USARTx);
		
		rxFsm(chr);
		
		USART_ClearITPendingBit(USARTx, USART_IT_RXNE);
	}
	
}


